#include "math.h"
#include "shader.h"

struct JS_fisheye {
  miScalar distortion;
  miScalar xscale;
  miScalar yscale;
  miScalar zscale;
  miBoolean mask;
};


DLLEXPORT int JS_fisheye_version(void)
{

	return 1;
}


DLLEXPORT miBoolean JS_fisheye
(
        miColor				*result,
        miState				*state,
        struct JS_fisheye   *paras)
    {
        miVector        camdir, dir;
        miScalar        x, y, r, t;
		miBoolean		m;

        mi_vector_to_camera(state, &camdir, &state->dir);
        t = state->camera->focal / -camdir.z /
                                (state->camera->aperture/2);
        x = t * camdir.x;
        y = t * camdir.y * state->camera->aspect;
		r = 0.99999999 * (1.0 - paras->distortion) + paras->distortion * (x * x + y * y);
        m = paras->mask;
		
		if (r < 1) {
                dir.x = camdir.x * r;
				dir.x = dir.x * paras->xscale;
                dir.y = camdir.y * r;
				dir.y = dir.y * paras->yscale;
                dir.z = -sqrt(1 - dir.x*dir.x - dir.y*dir.y);
				dir.z = dir.z * paras->zscale;
                mi_vector_from_camera(state, &dir, &dir);
                return(mi_trace_eye(result, state, &state->org, &dir));
        } else 
		
			if (m == 1)
			
			{
                result->r = result->g =
                result->b = result->a = 0;
                return(miFALSE);
			}
			else
		
			{
                dir.x = camdir.x * r;
				dir.x = dir.x * paras->xscale;
                dir.y = camdir.y * r;
				dir.y = dir.y * paras->yscale;
                dir.z = -sqrt(1 - dir.x*dir.x - dir.y*dir.y);
				dir.z = dir.z * paras->zscale;
                mi_vector_from_camera(state, &dir, &dir);
                return(mi_trace_eye(result, state, &state->org, &dir));
			}
    }

